@@ -1,56 +1,32 @@
-//! Collects _Tp-Note_'s configuration from a configuration file,
-//! the command line parameters. It also reads the clipboard.
+//! Set configuration defaults, reads and writes _Tp-Note_'s configuration file
+//! and exposes the configuration as `static` variable.
 
-use crate::content::Content;
 use crate::error::FileError;
-use crate::error::NoteError;
 use crate::filename;
+use crate::settings::ARGS;
 use crate::VERSION;
-use atty::{is, Stream};
-#[cfg(feature = "read-clipboard")]
-use clipboard::ClipboardContext;
-#[cfg(feature = "read-clipboard")]
-use clipboard::ClipboardProvider;
 use directories::ProjectDirs;
 use lazy_static::lazy_static;
 use log::LevelFilter;
-use parse_hyperlinks::iterator::first_hyperlink;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::fs;
-use std::io;
-use std::io::Read;
+#[cfg(not(test))]
+use std::fs::File;
+#[cfg(not(test))]
+use std::io::Write;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::RwLock;
-use structopt::StructOpt;
 
 /// Name of this executable (without the Windows ".exe" extension).
 const CURRENT_EXE: &str = "tp-note";
 
-/// Crate `confy` version 0.4 uses this filename by default.
+/// Tp-Note's configuration file filename.
 const CONFIG_FILENAME: &str = "tp-note.toml";
 
-/// Default value for command line option `--debug`.  Determines the maximum
-/// debug level events must have, to be logged.  If the command line option
-/// `--debug` is present, its value will be used instead.
-const DEBUG_ARG_DEFAULT: LevelFilter = LevelFilter::Error;
-
-/// Default value for command line flag `--edit` to disable file watcher,
-/// (Markdown)-renderer, html server and a web browser launcher set to `true`.
-const EDITOR_ARG_DEFAULT: bool = false;
-
-/// Default value for command line flag `--popup` If the command line flag
-/// `--popup` or `POPUP` is `true`, all log events will also trigger the
-/// appearance of a popup alert window.  Note, that error level debug events
-/// will always pop up, regardless of `--popup` and `POPUP` (unless
-/// `--debug=off`).
-const POPUP_ARG_DEFAULT: bool = true;
-
-/// Default value for command line flag `--no-filename-sync` to disable
-/// the title to filename synchronisation mechanism permanently.
-/// If set to `true`, the corresponding command line flag is ignored.
-const NO_FILENAME_SYNC_ARG_DEFAULT: bool = false;
+/// File extension of `to-note` files.
+pub const EXTENSION_DEFAULT: &str = "md";
 
 /// _Tp-Note_ opens all `.md` files with an external editor. It recognizes its
 /// own files, by the file extension `.md`, by a valid YAML header and the
@@ -62,8 +38,26 @@ const NO_FILENAME_SYNC_ARG_DEFAULT: bool = false;
 /// *) all string literals given in this example are configurable.
 const SILENTLY_IGNORE_MISSING_HEADER: bool = true;
 
-/// File extension of `to-note` files.
-pub const EXTENSION_DEFAULT: &str = "md";
+/// Default value for command line option `--debug`.  Determines the maximum
+/// debug level events must have, to be logged.  If the command line option
+/// `--debug` is present, its value will be used instead.
+const ARG_DEFAULT_DEBUG: LevelFilter = LevelFilter::Error;
+
+/// Default value for command line flag `--edit` to disable file watcher,
+/// (Markdown)-renderer, html server and a web browser launcher set to `true`.
+const ARG_DEFAULT_EDITOR: bool = false;
+
+/// Default value for command line flag `--no-filename-sync` to disable
+/// the title to filename synchronisation mechanism permanently.
+/// If set to `true`, the corresponding command line flag is ignored.
+const ARG_DEFAULT_NO_FILENAME_SYNC: bool = false;
+
+/// Default value for command line flag `--popup` If the command line flag
+/// `--popup` or `POPUP` is `true`, all log events will also trigger the
+/// appearance of a popup alert window.  Note, that error level debug events
+/// will always pop up, regardless of `--popup` and `POPUP` (unless
+/// `--debug=off`).
+const ARG_DEFAULT_POPUP: bool = true;
 
 /// The variables `NOTE_FILE_EXTENSIONS_*` list file extensions that Tp-Note
 /// considers as its own note files.
@@ -124,6 +118,15 @@ pub const NOTE_FILENAME_LEN_MAX: usize =
 #[cfg(test)]
 pub const NOTE_FILENAME_LEN_MAX: usize = 10;
 
+/// As all application logic is encoded in Tp-Note's templates, it does not know about field names.
+/// Nevertheless it is useful to identify at least one field as _the_ field that identifies a note
+/// the most.  When `TMPL_COMPULSORY_HEADER_FIELD` is not empty, Tp-Note will not synchronize the
+/// note's filename and will pop up an error message, unless it finds the field in the note's
+/// header.  When `TMPL_COMPULSORY_HEADER_FIELD` is empty, all files are synchronized without any
+/// further field check. Make sure to define a default value with `fm_* | default(value=*)`
+/// in case the variable `fm_*` does not exist in the note's front matter.
+const TMPL_COMPULSORY_HEADER_FIELD: &str = "title";
+
 /// Default content template used when the command line argument <sanit> is a directory. Can be
 /// changed through editing the configuration file.
 /// The following variables are  defined:
@@ -283,20 +286,12 @@ const TMPL_SYNC_FILENAME: &str = "\
 {{ fm_file_ext | default(value = path | ext) | prepend_dot }}\
 ";
 
-/// As all application logic is encoded in Tp-Note's templates, it does not know about field names.
-/// Nevertheless it is useful to identify at least one field as _the_ field that identifies a note
-/// the most.  When `TMPL_COMPULSORY_FIELD_CONTENT` is not empty, Tp-Note will not synchronize the
-/// note's filename and will pop up an error message, unless it finds the field in the note's
-/// header.  When `TMPL_COMPULSORY_FIELD_CONTENT` is empty, all files are synchronized without any
-/// further field check. Make sure to define a default value with `fm_* | default(value=*)`
-/// in case the variable `fm_*` does not exist in the note's front matter.
-const TMPL_COMPULSORY_FIELD_CONTENT: &str = "title";
 
 /// Default command line argument list when launching external editor.
 /// The editor list is executed item by item until an editor is found.
 /// Can be changed in config file.
 #[cfg(all(target_family = "unix", not(target_vendor = "apple")))]
-const EDITOR_ARGS: &[&[&str]] = &[
+const APP_ARGS_EDITOR: &[&[&str]] = &[
     &["code", "-w", "-n"],
     &["flatpak", "run", "com.visualstudio.code", "-w", "-n"],
     &["atom", "-w"],
@@ -317,7 +312,7 @@ const EDITOR_ARGS: &[&[&str]] = &[
     &["gvim", "--nofork"],
 ];
 #[cfg(target_family = "windows")]
-const EDITOR_ARGS: &[&[&str]] = &[
+const APP_ARGS_EDITOR: &[&[&str]] = &[
     &["C:\\Program Files\\Typora\\Typora.exe"],
     &[
         "C:\\Program Files\\Mark Text\\Mark Text.exe",
@@ -333,7 +328,7 @@ const EDITOR_ARGS: &[&[&str]] = &[
 // Some info about launching programs on iOS:
 //[dshell.pdf](https://www.stata.com/manuals13/dshell.pdf)
 #[cfg(all(target_family = "unix", target_vendor = "apple"))]
-const EDITOR_ARGS: &[&[&str]] = &[
+const APP_ARGS_EDITOR: &[&[&str]] = &[
     &["code", "-w", "-n"],
     &["atom", "-w"],
     &["marktext", "--no-sandbox"],
@@ -351,16 +346,16 @@ const EDITOR_ARGS: &[&[&str]] = &[
 /// The editor list is executed item by item until an editor is found.
 /// Can be changed in config file.
 #[cfg(all(target_family = "unix", not(target_vendor = "apple")))]
-const EDITOR_CONSOLE_ARGS: &[&[&str]] = &[&["nano"], &["nvim"], &["vim"], &["emacs"], &["vi"]];
+const APP_ARGS_EDITOR_CONSOLE: &[&[&str]] = &[&["nvim"], &["nano"], &["vim"], &["emacs"], &["vi"]];
 #[cfg(target_family = "windows")]
-const EDITOR_CONSOLE_ARGS: &[&[&str]] = &[&[]];
+const APP_ARGS_EDITOR_CONSOLE: &[&[&str]] = &[&[]];
 // Some info about launching programs on iOS:
 // [dshell.pdf](https://www.stata.com/manuals13/dshell.pdf)
 #[cfg(all(target_family = "unix", target_vendor = "apple"))]
-const EDITOR_CONSOLE_ARGS: &[&[&str]] = &[
+const APP_ARGS_EDITOR_CONSOLE: &[&[&str]] = &[
+    &["nvim"],
     &["nano"],
     &["pico"],
-    &["nvim"],
     &["vim"],
     &["emacs"],
     &["vi"],
@@ -370,7 +365,7 @@ const EDITOR_CONSOLE_ARGS: &[&[&str]] = &[
 /// The list is executed item by item until an installed web browser is found.
 /// Can be changed in config file.
 #[cfg(all(target_family = "unix", not(target_vendor = "apple")))]
-const BROWSER_ARGS: &[&[&str]] = &[
+const APP_ARGS_BROWSER: &[&[&str]] = &[
     &[
         "flatpak",
         "run",
@@ -398,7 +393,7 @@ const BROWSER_ARGS: &[&[&str]] = &[
     &["chrome", "--new-window", "--incognito"],
 ];
 #[cfg(target_family = "windows")]
-const BROWSER_ARGS: &[&[&str]] = &[
+const APP_ARGS_BROWSER: &[&[&str]] = &[
     &[
         "C:\\Program Files\\Mozilla Firefox\\firefox.exe",
         "--new-window",
@@ -417,7 +412,7 @@ const BROWSER_ARGS: &[&[&str]] = &[
 // Some info about launching programs on iOS:
 //[dshell.pdf](https://www.stata.com/manuals13/dshell.pdf)
 #[cfg(all(target_family = "unix", target_vendor = "apple"))]
-const BROWSER_ARGS: &[&[&str]] = &[];
+const APP_ARGS_BROWSER: &[&[&str]] = &[];
 
 /// By default clipboard support is enabled, can be disabled
 /// in config file. A false value here will set ENABLE_EMPTY_CLIPBOARD to
@@ -452,60 +447,44 @@ const COPY_COUNTER_CLOSING_BRACKETS: &str = ")";
 /// at the end.
 pub const COPY_COUNTER_MAX: usize = 400;
 
-/// When set to true, the viewer feature is automatically disabled when
-/// _Tp-Note_ encounters an `.md` file without header.  Experienced users can
-/// set this to `true`. See also `SILENTLY_IGNORE_MISSING_HEADER`.
-const VIEWER_MISSING_HEADER_DISABLES: bool = false;
+/// HTML template to render the viewer-error page.
+pub const VIEWER_ERROR_TMPL: &str = r#"<!DOCTYPE html>
+<html lang="en">
+<head>
+<meta charset="UTF-8">
+<title>Syntax error</title>
+<style>
+.note-error { color: #523626; }
+pre { white-space: pre-wrap; }
+a { color: #316128; }
+h1, h2, h3, h4, h5, h6 { color: #d3af2c; font-family:sans-serif; }
+</style>
+</head>
+<body>
+<h3>Syntax error</h3>
+<p> in note file: <pre>{{ path }}</pre><p>
+<div class="note-error">
+<hr>
+<pre>{{ note_error }}</pre>
+<hr>
+</div>
+{{ note_erroneous_content }}
+<script>{{ note_js }}</script>
+</body>
+</html>
+"#;
 
 /// How often should the file watcher check for changes?
 /// Delay in milliseconds.
 const VIEWER_NOTIFY_PERIOD: u64 = 1000;
 
-/// The maximum number of TCP connections the HTTP server can handle at the same
-/// time. In general, the serving and live update of the HTML rendition of the
-/// note file, requires normally 3 TCP connections: 1 old event channel (that is
-/// still open from the previous update), 1 TCP connection to serve the HTML,
-/// the local images (and referenced documents), and 1 new event channel.  In
-/// practise, stale connection are not always closed immediately. Hence 4 open
-/// connections are not uncommon.
-const VIEWER_TCP_CONNECTIONS_MAX: usize = 16;
+/// Template used by the viewer to render error messages into html.
+/// When set to true, the viewer feature is automatically disabled when
+/// _Tp-Note_ encounters an `.md` file without header.  Experienced users can
+/// set this to `true`. See also `SILENTLY_IGNORE_MISSING_HEADER`.
+const VIEWER_MISSING_HEADER_DISABLES: bool = false;
 
-/// Served file types with corresponding mime types.
-/// First entry per line is the file extension, the second the corresponding mime
-/// type. Embedded files with types other than those listed here are silently
-/// ignored.
-/// Note, that image files must be located in the same directory than the note
-/// file to be served.
-const VIEWER_SERVED_MIME_TYPES: &[&[&str]] = &[
-    &["apng", "image/apng"],
-    &["avif", "image/avif"],
-    &["bmp", "image/bmp"],
-    &["gif", "image/gif"],
-    &["html", "text/html"],
-    &["htm", "text/html"],
-    &["ico", "image/vnd.microsoft.icon"],
-    &["jpeg", "image/jpeg"],
-    &["jpg", "image/jpeg"],
-    &["pdf", "application/pdf"],
-    &["png", "image/png"],
-    &["svg", "image/svg+xml"],
-    &["tiff", "image/tiff"],
-    &["tif", "image/tiff"],
-    &["webp", "image/webp"],
-    &["mp3", "audio/mp3"],
-    &["ogg", "audio/ogg"],
-    &["oga", "audio/ogg"],
-    &["weba", "audio/webm"],
-    &["flac", "audio/flac"],
-    &["wav", "audio/wav"],
-    &["opus", "audio/opus"],
-    &["mp4", "video/mp4"],
-    &["ogv", "video/ogg"],
-    &["webm", "video/webm"],
-    &["ogx", "application/ogg"],
-];
-
-/// Template used by the viewer to render a note into html.
+/// Served file types with corresponding mime types.  First entry per line is
 pub const VIEWER_RENDITION_TMPL: &str = r#"<!DOCTYPE html>
 <html lang="{{ fm_lang | default(value='en') }}">
 <head>
@@ -570,32 +549,48 @@ h1, h2, h3, h4, h5, h6 { color: #263292; font-family:sans-serif; }
 </html>
 "#;
 
-/// Template used by the viewer to render error messages into html.
-pub const VIEWER_ERROR_TMPL: &str = r#"<!DOCTYPE html>
-<html lang="en">
-<head>
-<meta charset="UTF-8">
-<title>Syntax error</title>
-<style>
-.note-error { color: #523626; }
-pre { white-space: pre-wrap; }
-a { color: #316128; }
-h1, h2, h3, h4, h5, h6 { color: #d3af2c; font-family:sans-serif; }
-</style>
-</head>
-<body>
-<h3>Syntax error</h3>
-<p> in note file: <pre>{{ path }}</pre><p>
-<div class="note-error">
-<hr>
-<pre>{{ note_error }}</pre>
-<hr>
-</div>
-{{ note_erroneous_content }}
-<script>{{ note_js }}</script>
-</body>
-</html>
-"#;
+/// the file extension in lowercase, the second the corresponding mime type.
+/// Embedded files with types other than those listed here are silently ignored.
+/// Note, that image files must be located in the same or in the note's parent
+/// directory.
+const VIEWER_SERVED_MIME_TYPES: &[&[&str]] = &[
+    &["apng", "image/apng"],
+    &["avif", "image/avif"],
+    &["bmp", "image/bmp"],
+    &["gif", "image/gif"],
+    &["html", "text/html"],
+    &["htm", "text/html"],
+    &["ico", "image/vnd.microsoft.icon"],
+    &["jpeg", "image/jpeg"],
+    &["jpg", "image/jpeg"],
+    &["pdf", "application/pdf"],
+    &["png", "image/png"],
+    &["svg", "image/svg+xml"],
+    &["tiff", "image/tiff"],
+    &["tif", "image/tiff"],
+    &["webp", "image/webp"],
+    &["mp3", "audio/mp3"],
+    &["ogg", "audio/ogg"],
+    &["oga", "audio/ogg"],
+    &["weba", "audio/webm"],
+    &["flac", "audio/flac"],
+    &["wav", "audio/wav"],
+    &["opus", "audio/opus"],
+    &["mp4", "video/mp4"],
+    &["ogv", "video/ogg"],
+    &["webm", "video/webm"],
+    &["ogx", "application/ogg"],
+];
+
+/// Template used by the viewer to render a note into html.
+/// The maximum number of TCP connections the HTTP server can handle at the same
+/// time. In general, the serving and live update of the HTML rendition of the
+/// note file, requires normally 3 TCP connections: 1 old event channel (that is
+/// still open from the previous update), 1 TCP connection to serve the HTML,
+/// the local images (and referenced documents), and 1 new event channel.  In
+/// practise, stale connection are not always closed immediately. Hence 4 open
+/// connections are not uncommon.
+const VIEWER_TCP_CONNECTIONS_MAX: usize = 16;
 
 /// Template used to render a note into html when the
 /// rendition is saved to disk
@@ -662,65 +657,6 @@ h1, h2, h3, h4, h5, h6 { color: #263292; font-family:sans-serif; }
 </html>
 "#;
 
-#[derive(Debug, PartialEq, StructOpt)]
-#[structopt(
-    name = "Tp-Note",
-    about = "Fast note taking with templates and filename synchronization."
-)]
-/// `Tp-Note` is a note-taking-tool and a template system, that consistently
-/// synchronizes the note's meta-data with its filename. `tp-note` collects
-/// various information about its environment and the clipboard and stores them
-/// in variables. New notes are created by filling these variables in predefined
-/// and customizable `Tera`-templates. In case `<path>` points to an existing
-/// `tp-note`-file, the note's meta-data is analysed and, if necessary, its
-/// filename is modified. For all other file types, `tp-note` creates a new note
-/// that annotates the file `<path>` points to. If `<path>` is a directory (or,
-/// when omitted the current working directory), a new note is created in that
-/// directory. After creation, `tp-note` launches an external editor of your
-/// choice. Although the note's structure follows `pandoc`-conventions, it is not
-/// tied to any specific markup language.
-pub struct Args {
-    /// Batch made: does not launch editor or viewer
-    #[structopt(long, short = "b")]
-    pub batch: bool,
-    /// Loads alternative configuration file
-    #[structopt(long, short = "c")]
-    pub config: Option<String>,
-    /// Console debug level: "trace", "debug", "info", "warn", "error" (default) or "off"
-    #[structopt(long, short = "d")]
-    pub debug: Option<LevelFilter>,
-    /// Show console debug messages also as popup windows
-    #[structopt(long, short = "u")]
-    pub popup: bool,
-    /// Launches only the editor, no browser
-    #[structopt(long, short = "e")]
-    pub edit: bool,
-    /// Lets web server listen to a specific port
-    #[structopt(long, short = "p")]
-    pub port: Option<u16>,
-    /// Disables filename synchronization
-    #[structopt(long, short = "n")]
-    pub no_filename_sync: bool,
-    /// Launches only the browser, no editor
-    #[structopt(long, short = "v")]
-    pub view: bool,
-    /// <dir> as new note location or <file> to annotate
-    #[structopt(name = "PATH", parse(from_os_str))]
-    pub path: Option<PathBuf>,
-    /// Prints version and exits
-    #[structopt(long, short = "V")]
-    pub version: bool,
-    /// Saves the HTML rendition in the <export>
-    /// dir, the note's dir if '' or stdout if '-'.
-    #[structopt(long, short = "x", parse(from_os_str))]
-    pub export: Option<PathBuf>,
-}
-
-lazy_static! {
-/// Structure to hold the parsed command line arguments.
-pub static ref ARGS : Args = Args::from_args();
-}
-
 /// Configuration data, deserialized from the configuration file.
 #[derive(Debug, Serialize, Deserialize)]
 pub struct Cfg {
@@ -728,42 +664,95 @@ pub struct Cfg {
     /// a text message explaining why we could not load the
     /// configuration file.
     pub version: String,
-    pub debug_arg_default: LevelFilter,
-    pub edit_arg_default: bool,
-    pub no_filename_sync_arg_default: bool,
-    pub popup_arg_default: bool,
-    pub silently_ignore_missing_header: bool,
     pub extension_default: String,
-    pub note_file_extensions_md: Vec<String>,
-    pub note_file_extensions_rst: Vec<String>,
-    pub note_file_extensions_html: Vec<String>,
-    pub note_file_extensions_txt: Vec<String>,
-    pub note_file_extensions_no_viewer: Vec<String>,
-    pub tmpl_new_content: String,
-    pub tmpl_new_filename: String,
-    pub tmpl_copy_content: String,
-    pub tmpl_copy_filename: String,
-    pub tmpl_clipboard_content: String,
-    pub tmpl_clipboard_filename: String,
-    pub tmpl_annotate_content: String,
-    pub tmpl_annotate_filename: String,
-    pub tmpl_sync_filename: String,
-    pub tmpl_compulsory_field_content: String,
-    pub editor_args: Vec<Vec<String>>,
-    pub editor_console_args: Vec<Vec<String>>,
-    pub browser_args: Vec<Vec<String>>,
-    pub clipboard_read_enabled: bool,
-    pub clipboard_empty_enabled: bool,
-    pub copy_counter_extra_separator: String,
-    pub copy_counter_opening_brackets: String,
-    pub copy_counter_closing_brackets: String,
-    pub viewer_missing_header_disables: bool,
-    pub viewer_notify_period: u64,
-    pub viewer_tcp_connections_max: usize,
-    pub viewer_served_mime_types: Vec<Vec<String>>,
-    pub viewer_rendition_tmpl: String,
-    pub viewer_error_tmpl: String,
-    pub exporter_rendition_tmpl: String,
+    pub silently_ignore_missing_header: bool,
+    pub arg_default: ArgDefault,
+    pub note_file_extensions: NoteFileExtensions,
+    pub tmpl: Tmpl,
+    pub app_args: AppArgs,
+    pub clipboard: Clipboard,
+    pub copy_counter: CopyCounter,
+    pub viewer: Viewer,
+    pub exporter: Exporter,
+}
+
+/// Command line arguments, deserialized form configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct ArgDefault {
+    pub debug: LevelFilter,
+    pub edit: bool,
+    pub no_filename_sync: bool,
+    pub popup: bool,
+}
+
+/// Known note file extensions, deserialized from the configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct NoteFileExtensions {
+    pub md: Vec<String>,
+    pub rst: Vec<String>,
+    pub html: Vec<String>,
+    pub txt: Vec<String>,
+    pub no_viewer: Vec<String>,
+}
+
+/// Filename templates and content templates, deserialized from the configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct Tmpl {
+    pub compulsory_header_field: String,
+    pub new_content: String,
+    pub new_filename: String,
+    pub copy_content: String,
+    pub copy_filename: String,
+    pub clipboard_content: String,
+    pub clipboard_filename: String,
+    pub annotate_content: String,
+    pub annotate_filename: String,
+    pub sync_filename: String,
+}
+
+/// Arguments lists for invoking external applications, deserialized from the
+/// configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct AppArgs {
+    pub editor: Vec<Vec<String>>,
+    pub editor_console: Vec<Vec<String>>,
+    pub browser: Vec<Vec<String>>,
+}
+
+/// Configuration of clipboard behaviour, deserialized from the
+/// configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct Clipboard {
+    pub read_enabled: bool,
+    pub empty_enabled: bool,
+}
+
+/// Configuration of clipboard behaviour, deserialized from the
+/// configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct CopyCounter {
+    pub extra_separator: String,
+    pub opening_brackets: String,
+    pub closing_brackets: String,
+}
+
+/// Configuration data for the viewer feature, deserialized from the
+/// configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct Viewer {
+    pub error_tmpl: String,
+    pub notify_period: u64,
+    pub missing_header_disables: bool,
+    pub rendition_tmpl: String,
+    pub served_mime_types: Vec<Vec<String>>,
+    pub tcp_connections_max: usize,
+}
+
+/// Configuration for the HTML exporter feature, deserialized from the
+/// configuration file.
+#[derive(Debug, Serialize, Deserialize)]
+pub struct Exporter {
+    pub rendition_tmpl: String,
 }
 
 /// When no configuration file is found, defaults are set here from built-in
@@ -778,69 +767,141 @@ impl ::std::default::Default for Cfg {
 
         Cfg {
             version,
-            debug_arg_default: DEBUG_ARG_DEFAULT,
-            edit_arg_default: EDITOR_ARG_DEFAULT,
-            popup_arg_default: POPUP_ARG_DEFAULT,
-            no_filename_sync_arg_default: NO_FILENAME_SYNC_ARG_DEFAULT,
-            silently_ignore_missing_header: SILENTLY_IGNORE_MISSING_HEADER,
             extension_default: EXTENSION_DEFAULT.to_string(),
-            note_file_extensions_md: NOTE_FILE_EXTENSIONS_MD
+            silently_ignore_missing_header: SILENTLY_IGNORE_MISSING_HEADER,
+            arg_default: ArgDefault::default(),
+            note_file_extensions: NoteFileExtensions::default(),
+            tmpl: Tmpl::default(),
+            app_args: AppArgs::default(),
+            clipboard: Clipboard::default(),
+            copy_counter: CopyCounter::default(),
+            viewer: Viewer::default(),
+            exporter: Exporter::default(),
+        }
+    }
+}
+
+/// Default values for command line arguments.
+impl ::std::default::Default for ArgDefault {
+    fn default() -> Self {
+        ArgDefault {
+            debug: ARG_DEFAULT_DEBUG,
+            edit: ARG_DEFAULT_EDITOR,
+            no_filename_sync: ARG_DEFAULT_NO_FILENAME_SYNC,
+            popup: ARG_DEFAULT_POPUP,
+        }
+    }
+}
+
+/// Default values for known note file extensions.
+impl ::std::default::Default for NoteFileExtensions {
+    fn default() -> Self {
+        NoteFileExtensions {
+            md: NOTE_FILE_EXTENSIONS_MD
                 .iter()
                 .map(|a| (*a).to_string())
                 .collect(),
-            note_file_extensions_rst: NOTE_FILE_EXTENSIONS_RST
+            rst: NOTE_FILE_EXTENSIONS_RST
                 .iter()
                 .map(|a| (*a).to_string())
                 .collect(),
-            note_file_extensions_html: NOTE_FILE_EXTENSIONS_HTML
+            html: NOTE_FILE_EXTENSIONS_HTML
                 .iter()
                 .map(|a| (*a).to_string())
                 .collect(),
-            note_file_extensions_txt: NOTE_FILE_EXTENSIONS_TXT
+            txt: NOTE_FILE_EXTENSIONS_TXT
                 .iter()
                 .map(|a| (*a).to_string())
                 .collect(),
-            note_file_extensions_no_viewer: NOTE_FILE_EXTENSIONS_NO_VIEWER
+            no_viewer: NOTE_FILE_EXTENSIONS_NO_VIEWER
                 .iter()
                 .map(|a| (*a).to_string())
                 .collect(),
-            tmpl_new_content: TMPL_NEW_CONTENT.to_string(),
-            tmpl_new_filename: TMPL_NEW_FILENAME.to_string(),
-            tmpl_copy_content: TMPL_COPY_CONTENT.to_string(),
-            tmpl_copy_filename: TMPL_COPY_FILENAME.to_string(),
-            tmpl_clipboard_content: TMPL_CLIPBOARD_CONTENT.to_string(),
-            tmpl_clipboard_filename: TMPL_CLIPBOARD_FILENAME.to_string(),
-            tmpl_annotate_content: TMPL_ANNOTATE_CONTENT.to_string(),
-            tmpl_annotate_filename: TMPL_ANNOTATE_FILENAME.to_string(),
-            tmpl_sync_filename: TMPL_SYNC_FILENAME.to_string(),
-            tmpl_compulsory_field_content: TMPL_COMPULSORY_FIELD_CONTENT.to_string(),
-            editor_args: EDITOR_ARGS
+        }
+    }
+}
+
+/// Default values for templates.
+impl ::std::default::Default for Tmpl {
+    fn default() -> Self {
+        Tmpl {
+            compulsory_header_field: TMPL_COMPULSORY_HEADER_FIELD.to_string(),
+            new_content: TMPL_NEW_CONTENT.to_string(),
+            new_filename: TMPL_NEW_FILENAME.to_string(),
+            copy_content: TMPL_COPY_CONTENT.to_string(),
+            copy_filename: TMPL_COPY_FILENAME.to_string(),
+            clipboard_content: TMPL_CLIPBOARD_CONTENT.to_string(),
+            clipboard_filename: TMPL_CLIPBOARD_FILENAME.to_string(),
+            annotate_content: TMPL_ANNOTATE_CONTENT.to_string(),
+            annotate_filename: TMPL_ANNOTATE_FILENAME.to_string(),
+            sync_filename: TMPL_SYNC_FILENAME.to_string(),
+        }
+    }
+}
+
+/// Default values for invoking external applications.
+impl ::std::default::Default for AppArgs {
+    fn default() -> Self {
+        AppArgs {
+            editor: APP_ARGS_EDITOR
                 .iter()
                 .map(|i| i.iter().map(|a| (*a).to_string()).collect())
                 .collect(),
-            editor_console_args: EDITOR_CONSOLE_ARGS
+            editor_console: APP_ARGS_EDITOR_CONSOLE
                 .iter()
                 .map(|i| i.iter().map(|a| (*a).to_string()).collect())
                 .collect(),
-            browser_args: BROWSER_ARGS
+            browser: APP_ARGS_BROWSER
                 .iter()
                 .map(|i| i.iter().map(|a| (*a).to_string()).collect())
                 .collect(),
-            clipboard_read_enabled: CLIPBOARD_READ_ENABLED,
-            clipboard_empty_enabled: CLIPBOARD_EMPTY_ENABLED,
-            copy_counter_extra_separator: COPY_COUNTER_EXTRA_SEPARATOR.to_string(),
-            copy_counter_opening_brackets: COPY_COUNTER_OPENING_BRACKETS.to_string(),
-            copy_counter_closing_brackets: COPY_COUNTER_CLOSING_BRACKETS.to_string(),
-            viewer_missing_header_disables: VIEWER_MISSING_HEADER_DISABLES,
-            viewer_notify_period: VIEWER_NOTIFY_PERIOD,
-            viewer_tcp_connections_max: VIEWER_TCP_CONNECTIONS_MAX,
-            viewer_served_mime_types: VIEWER_SERVED_MIME_TYPES
+        }
+    }
+}
+
+/// Default values for clipboard behaviour.
+impl ::std::default::Default for Clipboard {
+    fn default() -> Self {
+        Clipboard {
+            read_enabled: CLIPBOARD_READ_ENABLED,
+            empty_enabled: CLIPBOARD_EMPTY_ENABLED,
+        }
+    }
+}
+
+/// Default values for copy counter.
+impl ::std::default::Default for CopyCounter {
+    fn default() -> Self {
+        CopyCounter {
+            extra_separator: COPY_COUNTER_EXTRA_SEPARATOR.to_string(),
+            opening_brackets: COPY_COUNTER_OPENING_BRACKETS.to_string(),
+            closing_brackets: COPY_COUNTER_CLOSING_BRACKETS.to_string(),
+        }
+    }
+}
+
+/// Default values for the viewer feature.
+impl ::std::default::Default for Viewer {
+    fn default() -> Self {
+        Viewer {
+            missing_header_disables: VIEWER_MISSING_HEADER_DISABLES,
+            notify_period: VIEWER_NOTIFY_PERIOD,
+            tcp_connections_max: VIEWER_TCP_CONNECTIONS_MAX,
+            served_mime_types: VIEWER_SERVED_MIME_TYPES
                 .iter()
                 .map(|i| i.iter().map(|a| (*a).to_string()).collect())
                 .collect(),
-            viewer_rendition_tmpl: VIEWER_RENDITION_TMPL.to_string(),
-            viewer_error_tmpl: VIEWER_ERROR_TMPL.to_string(),
-            exporter_rendition_tmpl: EXPORTER_RENDITION_TMPL.to_string(),
+            rendition_tmpl: VIEWER_RENDITION_TMPL.to_string(),
+            error_tmpl: VIEWER_ERROR_TMPL.to_string(),
+        }
+    }
+}
+
+/// Default values for the exporter feature.
+impl ::std::default::Default for Exporter {
+    fn default() -> Self {
+        Exporter {
+            rendition_tmpl: EXPORTER_RENDITION_TMPL.to_string(),
         }
     }
 }
@@ -849,7 +910,7 @@ lazy_static! {
     /// Store the extension as key and mime type as value in HashMap.
     pub static ref VIEWER_SERVED_MIME_TYPES_HMAP: HashMap<&'static str, &'static str> = {
         let mut hm = HashMap::new();
-        for l in &CFG.viewer_served_mime_types {
+        for l in &CFG.viewer.served_mime_types {
             if l.len() >= 2
             {
                 hm.insert(l[0].as_str(), l[1].as_str());
@@ -859,61 +920,38 @@ lazy_static! {
     };
 }
 
-lazy_static! {
-    /// Shall we launch the external text editor?
-    pub static ref LAUNCH_EDITOR: bool = {
-        !ARGS.batch && ARGS.export.is_none() && !ARGS.view
-    };
-}
-
-#[cfg(feature = "viewer")]
-lazy_static! {
-    /// Shall we launch the internal http server and the external browser?
-    pub static ref LAUNCH_VIEWER: bool = {
-        !ARGS.batch && ARGS.export.is_none() && !*RUNS_ON_CONSOLE &&
-            (ARGS.view || ( !ARGS.edit && !CFG.edit_arg_default ))
-    };
-}
-
-#[cfg(not(feature = "viewer"))]
-lazy_static! {
-    /// Shall we launch the internal http server and the external browser?
-    pub static ref LAUNCH_VIEWER: bool = {
-        false
-    };
-}
-
-lazy_static! {
-    /// Do we run on a console?
-    pub static ref RUNS_ON_CONSOLE: bool = {
-        // On Linux popup window only if DISPLAY is set.
-        #[cfg(target_family = "unix")]
-        let display = std::env::var("DISPLAY")
-            // Map error to `None`.
-            .ok()
-            // A pattern mapping `Some("")` to `None`.
-            .and_then(|s: String| if s.is_empty() { None } else { Some(s) });
-
-        // In non-Linux there is always "Some" display.
-        #[cfg(not(target_family = "unix"))]
-        let display = Some(String::new());
-
-        display.is_none()
-    };
-}
-
 lazy_static! {
     /// Variable indicating with `Err` if the loading of the configuration file went wrong.
     pub static ref CFG_FILE_LOADING: RwLock<Result<(), FileError>> = RwLock::new(Ok(()));
 }
 
+/// Parse the configuration file if it exists. Otherwise write one with default values.
 #[cfg(not(test))]
+#[inline]
+fn config_load_path(config_path: &Path) -> Result<Cfg, FileError> {
+    if config_path.exists() {
+        let config: Cfg = toml::from_str(&fs::read_to_string(config_path)?)?;
+        Ok(config)
+    } else {
+        let mut buffer = File::create(config_path)?;
+        buffer.write_all(toml::to_string_pretty(&Cfg::default())?.as_bytes())?;
+        Ok(Cfg::default())
+    }
+}
+
+/// In unit tests we use the default configuration values.
+#[cfg(test)]
+#[inline]
+fn config_load_path(_config_path: &Path) -> Result<Cfg, FileError> {
+    Ok(Cfg::default())
+}
+
 lazy_static! {
     /// Reads and parses the configuration file "tp-note.toml". An alternative
     /// filename (optionally with absolute path) can be given on the command line
     /// with "--config".
-    pub static ref CFG: Cfg = confy::load_path(&(
-        if let Some(c) = &ARGS.config {
+    pub static ref CFG: Cfg = {
+        let config_path = if let Some(c) = &ARGS.config {
             Path::new(c)
         } else {
             match &*CONFIG_PATH {
@@ -925,26 +963,23 @@ lazy_static! {
                     return Cfg::default();
                 },
             }
+        };
 
-        })
-        ).unwrap_or_else(|e|{
-            // Remember that something went wrong.
-            let mut cfg_file_loading = CFG_FILE_LOADING.write().unwrap();
-            *cfg_file_loading = Err(e.into());
+        config_load_path(&config_path)
+            .unwrap_or_else(|e|{
+                // Remember that something went wrong.
+                let mut cfg_file_loading = CFG_FILE_LOADING.write().unwrap();
+                *cfg_file_loading = Err(e.into());
 
-            // As we could not load the config file, we will use the default
-            // configuration.
-            Cfg::default()
-        });
-}
-
-#[cfg(test)]
-lazy_static! {
-    pub static ref CFG: Cfg = Cfg::default();
+                // As we could not load the config file, we will use the default
+                // configuration.
+                Cfg::default()
+            })
+        };
 }
 
 lazy_static! {
-/// This is where the `confy` crate stores the configuration file.
+/// This is where the Tp-Note stores its configuration file.
     pub static ref CONFIG_PATH : Option<PathBuf> = {
         if let Some(c) = &ARGS.config {
             Some(PathBuf::from(c))
@@ -973,119 +1008,3 @@ pub fn backup_config_file() -> Result<PathBuf, FileError> {
         Err(FileError::PathToConfigFileNotFound)
     }
 }
-
-lazy_static! {
-    /// Reads the input stream stdin if there is any.
-    pub static ref STDIN: Content = {
-        let mut buffer = String::new();
-
-        // Read stdin().
-        if !is(Stream::Stdin) {
-            let stdin = io::stdin();
-            let mut handle = stdin.lock();
-            let _ = handle.read_to_string(&mut buffer);
-        }
-
-        // `trim_end()` content without new allocation.
-        buffer.truncate(buffer.trim_end().len());
-
-        Content::from_input_with_cr(buffer)
-    };
-}
-
-lazy_static! {
-    /// Reads the clipboard, if there is any and empties it.
-    pub static ref CLIPBOARD: Content = {
-        let mut buffer = String::new();
-
-        // Concatenate clipboard content.
-        #[cfg(feature="read-clipboard")]
-        if CFG.clipboard_read_enabled && !*RUNS_ON_CONSOLE && !ARGS.batch {
-            let ctx: Option<ClipboardContext> = ClipboardProvider::new().ok();
-            if ctx.is_some() {
-                let ctx = &mut ctx.unwrap(); // This is ok since `is_some()>`
-                let s = ctx.get_contents().ok();
-                buffer.push_str(&s.unwrap_or_default());
-            }
-        };
-
-        // `trim_end()` content without new allocation.
-        buffer.truncate(buffer.trim_end().len());
-
-        Content::from_input_with_cr(buffer)
-    };
-}
-
-#[derive(Debug, PartialEq, Default)]
-/// Represents a hyperlink.
-pub struct Hyperlink {
-    pub name: String,
-    pub target: String,
-    pub title: String,
-}
-
-impl Hyperlink {
-    /// Parse a markdown formatted hyperlink and stores the result in `Self`.
-    pub fn new(input: &str) -> Result<Hyperlink, NoteError> {
-        if let Some((link_name, link_target, link_title)) = first_hyperlink(input) {
-            Ok(Hyperlink {
-                name: link_name.to_string(),
-                target: link_target.to_string(),
-                title: link_title.to_string(),
-            })
-        } else {
-            Err(NoteError::NoHyperlinkFound)
-        }
-    }
-}
-
-#[cfg(test)]
-mod tests {
-    use super::Hyperlink;
-
-    #[test]
-    fn test_parse_hyperlink() {
-        // Stand alone Markdown link.
-        let input = r#"abc[Homepage](https://blog.getreu.net "My blog")abc"#;
-        let expected_output = Hyperlink {
-            name: "Homepage".to_string(),
-            target: "https://blog.getreu.net".to_string(),
-            title: "My blog".to_string(),
-        };
-        let output = Hyperlink::new(input);
-        assert_eq!(expected_output, output.unwrap());
-
-        // Markdown link refernce.
-        let input = r#"abc[Homepage][home]abc
-                      [home]: https://blog.getreu.net "My blog""#;
-        let expected_output = Hyperlink {
-            name: "Homepage".to_string(),
-            target: "https://blog.getreu.net".to_string(),
-            title: "My blog".to_string(),
-        };
-        let output = Hyperlink::new(input);
-        assert_eq!(expected_output, output.unwrap());
-
-        //
-        // RestructuredText link
-        let input = "abc`Homepage <https://blog.getreu.net>`_\nabc";
-        let expected_output = Hyperlink {
-            name: "Homepage".to_string(),
-            target: "https://blog.getreu.net".to_string(),
-            title: "".to_string(),
-        };
-        let output = Hyperlink::new(input);
-        assert_eq!(expected_output, output.unwrap());
-
-        //
-        // RestructuredText link ref
-        let input = "abc `Homepage<home_>`_ abc\n.. _home: https://blog.getreu.net\nabc";
-        let expected_output = Hyperlink {
-            name: "Homepage".to_string(),
-            target: "https://blog.getreu.net".to_string(),
-            title: "".to_string(),
-        };
-        let output = Hyperlink::new(input);
-        assert_eq!(expected_output, output.unwrap());
-    }
-}
