@@ -1,7 +1,6 @@
 //! Custom error types.
 
 use crate::process_ext::ChildExtError;
-use confy::ConfyError;
 use std::io;
 use std::path::PathBuf;
 use std::process::ExitStatus;
@@ -107,14 +106,15 @@ pub enum FileError {
     #[error("Can not convert path to UFT8:\n{path:?}")]
     PathNotUtf8 { path: PathBuf },
 
-    /// Remedy: check the configuration file variables `editor_args` and `browser_args`.
+    /// Remedy: check the configuration file variables `[app_args] editor`
+    /// and `[app_args] browser`.
     #[error("Error executing external application:")]
     ChildExt {
         #[from]
         source: ChildExtError,
     },
 
-    /// Remedy: check the configuration file variable `editor_args`.
+    /// Remedy: check the configuration file variable `[app_args] editor`.
     #[error(
         "The external application did not terminate gracefully: {code}\n\
          \n\
@@ -128,7 +128,7 @@ pub enum FileError {
         args: Vec<String>,
     },
 
-    /// Remedy: check the configuration file variable `editor_args`.
+    /// Remedy: check the configuration file variable `[app_args] editor`.
     #[error(
         "None of the following external applications can be found on your system:\n\
         \t{app_list:?}\n\
@@ -146,7 +146,10 @@ pub enum FileError {
     Io(#[from] std::io::Error),
 
     #[error(transparent)]
-    Confy(#[from] ConfyError),
+    Serialize(#[from] toml::ser::Error),
+
+    #[error(transparent)]
+    Deserialize(#[from] toml::de::Error),
 }
 
 /// The error `InvalidFrontMatterYaml` prints the front matter section of the
@@ -260,7 +263,7 @@ pub enum NoteError {
         \tfile_ext=\"{extension}\"\n\
         \t---\n\
         is not registered as a valid\n\
-        Tp-Note-file in the `note_file_extensions_*` variables\n\
+        Tp-Note-file in the `note_file_extensions.*` variables\n\
         in your configuration file:\n\
         \t{md_ext:?}\n\
         \t{rst_ext:?}\n\
@@ -269,7 +272,7 @@ pub enum NoteError {
         \t{no_viewer_ext:?}\n\
         \n\
         Choose one of the above list or add more extensions to the\n\
-        `note_file_extensions_*` variables in your configuration file."
+        `note_file_extensions.*` variables in your configuration file."
     )]
     FileExtNotRegistered {
         extension: String,
